import random
import time
import datetime
from functools import wraps
from types import FunctionType

class TaskBase:
    # __init_subclass__方法在Python 3.6及之后的版本中被引入，当一个类被子类化时，这个方法就会被调用。
    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        for attr, value in cls.__dict__.items():
            if type(value) is FunctionType:
                setattr(cls, attr, cls.log_execution_time(value))

    @staticmethod
    # 自动埋点
    def log_execution_time(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            # 将参数转化为字符串并截取前16个字符
            str_args = ', '.join(str(arg)[:16] for arg in args[1:])  # args[0] is self
            str_kwargs = ', '.join(f'{k}={v}'[:16] for k, v in kwargs.items())
            params = ', '.join([str_args, str_kwargs]).strip(', ')
            
            start_time = datetime.datetime.now()
            start_time_str = start_time.strftime('%Y-%m-%d %H:%M:%S')

            result = None  # initialize result variable
            try:
                result = func(*args, **kwargs)
            except Exception as e:
                error_time_str = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                print(f"{error_time_str}\t{func.__qualname__}({params})出现异常:\t {str(e)}")
                # uncomment the following line if you want the error to be raised after being logged
                # raise

            end_time = datetime.datetime.now()
            time_taken = (end_time - start_time).total_seconds()
            time_taken_str = f"{time_taken:.3f}s"
            log_message = f"{start_time_str}\t{time_taken_str}\t{args[0].__class__.__base__.__name__}.{func.__qualname__}({params})"
            print(log_message)

            return result
        return wrapper

# class testSubClass(TaskBase):
#     def my_method(self, arg1, arg2, arg3):
#         time.sleep(random.uniform(0.1, 2))

#     def error_test(self):
#         time.sleep(random.uniform(0.1, 2))
#         try:
#             print(1/0)            
#         except Exception as e:
#             print(e)
#         print(1/0)
#         print("hello")

# a=testSubClass()
# a.error_test()

